<?php

/**
 * @file
 * Contains \Drupal\drd\Entity\DrdDomain.
 */

namespace Drupal\drd\Entity;

use Drupal\Core\Annotation\Translation;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\UseCacheBackendTrait;
use Drupal\Core\Entity\Annotation\ContentEntityType;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Routing\LinkGeneratorTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\drd\DrdDomainInterface;
use Drupal\drd\Plugin\DrdActionRemoteBase;
use Drupal\drd\Plugin\DrdAuthBase;
use Drupal\user\UserInterface;
use GuzzleHttp\Client;

/**
 * Defines the Domain entity.
 *
 * @ingroup drd
 *
 * @ContentEntityType(
 *   id = "drd_domain",
 *   label = @Translation("Domain"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\drd\DrdDomainListBuilder",
 *     "views_data" = "Drupal\drd\Entity\DrdDomainViewsData",
 *
 *     "form" = {
 *       "default" = "Drupal\drd\Form\DrdDomainForm",
 *       "edit" = "Drupal\drd\Form\DrdDomainForm",
 *     },
 *     "access" = "Drupal\drd\DrdDomainAccessControlHandler",
 *   },
 *   base_table = "drd_domain",
 *   admin_permission = "administer DrdDomain entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *   },
 *   links = {
 *     "canonical" = "/drd/domain/{drd_domain}",
 *     "edit-form" = "/drd/domain/{drd_domain}/edit",
 *   },
 *   field_ui_base_route = "drd_domain.settings"
 * )
 */
class DrdDomain extends ContentEntityBase implements DrdDomainInterface {
  use EntityChangedTrait;
  use LinkGeneratorTrait;
  use UseCacheBackendTrait;
  use StringTranslationTrait;

  # TODO: Drupal 6 and 7 default to FALSE, Drupal 8 only supports TRUE
  /**
   * @var bool
   */
  protected $cleanUrl = TRUE;

  /**
   * @inheritDoc
   */
  public function __construct(array $values, $entity_type, $bundle) {
    parent::__construct($values, $entity_type, $bundle);
    $this->cacheBackend = \Drupal::cache();
  }

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += ['user_id' => \Drupal::currentUser()->id(),];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function getCore() {
    return $this->get('core')->entity;
  }

  /**
   * @inheritDoc
   */
  public function setCore($core) {
    $this->set('core', $core->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? NODE_PUBLISHED : NODE_NOT_PUBLISHED);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getAuthSetting($type = NULL, $for_remote = FALSE) {
    if (!isset($type)) {
      $type = $this->get('auth')->value;
    }
    if (empty($type)) {
      return [];
    }

    if ($for_remote) {
      /** @var DrdAuthBase $auth */
      $auth = \Drupal::service('plugin.manager.drd_auth')->createInstance($type);
      if (!$auth->storeSettingRemotely()) {
        return [];
      }
    }
    $settings = $this->get('authsetting')->getValue()[0];
    return isset($settings[$type]) ? $settings[$type] : [];
  }

  /**
   * {@inheritdoc}
   */
  public function getCryptSetting($type = NULL) {
    if (!isset($type)) {
      $type = $this->get('crypt')->value;
    }
    if (empty($type)) {
      return [];
    }
    $settings = $this->get('cryptsetting')->getValue()[0];
    return isset($settings[$type]) ? $settings[$type] : [];
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('ID'))
      ->setDescription(t('The ID of the Domain entity.'))
      ->setReadOnly(TRUE);
    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setDescription(t('The UUID of the Domain entity.'))
      ->setReadOnly(TRUE);
    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Host entity.'))
      ->setReadOnly(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDefaultValueCallback('Drupal\node\Entity\Node::getCurrentUserId')
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name of the Domain entity.'))
      ->setSettings([
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -11,
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the Domain is published.'))
      ->setDefaultValue(TRUE);

    $fields['langcode'] = BaseFieldDefinition::create('language')
      ->setLabel(t('Language code'))
      ->setDescription(t('The language code for the Domain entity.'));

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    $fields['terms'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Tags'))
      ->setDescription(t('Tags for the host.'))
      ->setSetting('target_type', 'taxonomy_term')
      ->setSetting('handler', 'default:taxonomy_term')
      ->setSetting('handler_settings', [
        'target_bundles' => ['tags'],
        'sort' => ['field' => '_none'],
        'auto_create' => TRUE,
      ])
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setCustomStorage(FALSE)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete_tags',
        'weight' => -1,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'placeholder' => '',
        ],
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -1,
        'settings' => [
          'link' => TRUE,
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['core'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Core'))
      ->setDescription(t('The Drupal core that contains that domain.'))
      ->setSetting('target_type', 'drd_core')
      ->setRequired(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -10,
        'settings' => [
          'link' => TRUE,
        ],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['releases'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Releases'))
      ->setDescription(t('The installed core, module and theme releases on this domain.'))
      ->setSetting('target_type', 'drd_release')
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED);

    $fields['warnings'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Warnings'))
      ->setDescription(t('The requirements from the status page that cause a warning.'))
      ->setSetting('target_type', 'drd_requirement')
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED);

    $fields['errors'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Errors'))
      ->setDescription(t('The requirements from the status page that cause an error.'))
      ->setSetting('target_type', 'drd_requirement')
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED);

    $fields['domain'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Domain'))
      ->setDescription(t('The domain name.'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -7,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['aliase'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Aliase'))
      ->setDescription(t('The aliases for this domain name.'))
      ->setSettings([
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -4,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['path'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Path'))
      ->setDescription(t('The base path.'))
      ->setRequired(TRUE)
      ->setDefaultValue('')
      ->setSettings([
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -6,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['port'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Port'))
      ->setDescription(t('The port to connect to if different from default ports.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -5,
        'settings' => [
          'type' => 'number_unformatted',
        ],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['auth'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Authentication'))
      ->setDescription(t('The selected type of authenticaion.'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -3,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['authsetting'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Authentication Settings'))
      ->setDescription(t('Serialized settings for authentication.'))
      ->setDefaultValue([]);

    $fields['crypt'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Encryption'))
      ->setDescription(t('The selected type of encryption.'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -2,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['cryptsetting'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Encryption Settings'))
      ->setDescription(t('Serialized settings for encryption.'))
      ->setDefaultValue([]);

    $fields['installed'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Installed'))
      ->setDescription(t('A boolean indicating whether the DRD module is installed remotely.'))
      ->setDefaultValue(FALSE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -9,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['secure'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Secure connection'))
      ->setDescription(t('A boolean indicating whether SSL is supported.'))
      ->setDefaultValue(FALSE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => -8,
        'settings' => [],
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
   * @inheritDoc
   */
  public static function instanceFromUrl($core, $uri, $values) {
    $uri_parts = parse_url($uri);
    $storage = \Drupal::entityTypeManager()->getStorage('drd_domain');

    $query = \Drupal::entityQuery('drd_domain')
      ->condition('domain', $uri_parts['host']);
    if (isset($uri_parts['path'])) {
      $query->condition('path', $uri_parts['path']);
    }
    else {
      $query->notExists('path');
    }
    if (isset($uri_parts['port'])) {
      $query->condition('port', $uri_parts['port']);
    }
    $ids = $query->execute();

    if (empty($ids)) {
      if (!$core->isNew()) {
        $values['core'] = $core->id();
      }
      $values['domain'] = $uri_parts['host'];
      $values['path'] = isset($uri_parts['path']) ? $uri_parts['path'] : '';
      $values['secure'] = ($uri_parts['scheme'] == 'https');
      $values['port'] = isset($uri_parts['port']) ? $uri_parts['port'] : ($values['secure'] ? 443 : 80);
      $domain = $storage->create($values);
    }
    else {
      $domain = $storage->load(reset($ids));
    }
    return $domain;
  }

  /**
   * @inheritDoc
   */
  public function buildUrl($query = '') {
    $uri = $this->get('secure')->value ? 'https' : 'http';
    $uri .= '://' . $this->get('domain')->value;
    if (!$this->isDefaultPort()) {
      $uri .= ':' . $this->get('port')->value;
    }
    if (!empty($this->get('path')->value)) {
      $uri .= '/' . $this->get('path')->value;
    }
    if ($this->cleanUrl && !empty($query)) {
      $uri .= '/' . $query;
    }
    $url = Url::fromUri($uri);
    if (!$this->cleanUrl && !empty($query)) {
      $url->setOption('query', ['q' => $query,]);
    }
    return $url;
  }

  /**
   * @inheritDoc
   */
  public function getSupportedCryptMethods($cleanUrl = FALSE) {
    $this->cleanUrl = $cleanUrl;
    $url = $this->buildUrl('drd-remote-crypt');
    try {
      $client = new Client(['base_uri' => $url->toUriString()]);
      $response = $client->request('get');
    }
    catch (\Exception $ex) {
      return FALSE;
    }
    if ($response->getStatusCode() != 200 || $response->getHeaderLine('content-type') != 'text/plain; charset=utf-8') {
      return $cleanUrl ? '' : $this->getSupportedCryptMethods(TRUE);
    }
    return unserialize(base64_decode($response->getBody()->getContents()));
  }

  /**
   * @inheritDoc
   */
  public function pushOTT($token) {
    $url = $this->buildUrl('drd-remote');
    $body = base64_encode(serialize([
      'uuid' => $this->uuid(),
      'args' => 'none',
      'iv' => 'none',
      'ott' => $token,
      'config' => $this->getRemoteSetupToken(FALSE),
    ]));

    // Send request
    try {
      $client = new Client(['base_uri' => $url->toUriString()]);
      $response = $client->request('get', NULL, ['body' => $body]);
    }
    catch (\Exception $ex) {
      return FALSE;
    }
    if ($response->getStatusCode() != 200 || $response->getHeaderLine('content-type') != 'text/plain; charset=utf-8') {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * @inheritDoc
   */
  public function isDefaultPort() {
    $secure = $this->get('secure')->value;
    $port = $this->get('port')->value;
    return $secure ? ($port == 443) : ($port == 80);
  }

  /**
   * @inheritDoc
   */
  public function getRemoteSetupToken($redirect) {
    $values = [
      'uuid' => $this->get('uuid')->value,
      'auth' => $this->get('auth')->value,
      'authsetting' => $this->getAuthSetting(NULL, TRUE),
      'crypt' => $this->get('crypt')->value,
      'cryptsetting' => $this->getCryptSetting(),
      'redirect' => $redirect,
    ];

    $values = base64_encode(serialize($values));
    return strtr($values, array('+' => '-', '/' => '_', '=' => ''));
  }

  /**
   * @inheritDoc
   */
  public function getRemoteSetupLink($label, $initial = FALSE) {
    if ($initial) {
      $redirect = new Url('entity.drd_domain.return_remote', ['domain' => $this->id(),], ['absolute' => TRUE]);
      $redirect->setOption('query', ['token' => \Drupal::csrfToken()->get($redirect->getInternalPath())]);
    }
    else {
      $redirect = new Url('entity.drd_core.collection', [], ['absolute' => TRUE]);
    }

    $url = $this->buildUrl('drd-remote-setup/' . $this->getRemoteSetupToken($redirect->toString()));
    return $this->l($label, $url);
  }

  /**
   * @inheritDoc
   */
  public function ping() {
    return DrdActionRemoteBase::response('drd_action_ping', $this);
  }

  /**
   * @inheritDoc
   */
  public function remoteInfo() {
    return DrdActionRemoteBase::response('drd_action_info', $this);
  }

  /**
   * @inheritDoc
   */
  public function initCore($core) {
    $response = $this->remoteInfo();

    if (!$response) {
      return FALSE;
    }

    // Set Drupal root directory
    $core->set('drupalroot', $response['root']);

    // Set Drupal version
    $release = DrdRelease::findOrCreate('core', 'drupal', $response['version']);
    $core->set('drupalversion', $release->id());

    // Save core
    $core->save();

    return TRUE;
  }

  /**
   * @inheritDoc
   */
  public function initValues($name, $crypt = NULL, $crypt_setting = []) {
    $this->setName($name);
    // Initialize settings
    if (empty($crypt)) {
      $crypt = 'OpenSSL';
      $crypt_setting = ['cipher' => 'aes-128-cbc'];
    }
    $crypt_setting['password'] = user_password(50);
    $this->set('auth', 'shared_secret');
    $this->set('authsetting', ['shared_secret' => ['secret' => user_password(50)]]);
    $this->set('crypt', $crypt);
    $this->set('cryptsetting', [$crypt => $crypt_setting]);
  }

  /**
   * @inheritDoc
   */
  public function retrieveAllDomains($core) {
    $this->set('core', $core->id());
    $this->save();
    DrdActionRemoteBase::response('drd_action_domains_receive', $core);
  }

  /**
   * @inheritDoc
   */
  public function setAliase($aliase) {
    $this->set('aliase', $aliase);
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function getReleases() {
    return \Drupal::entityTypeManager()->getStorage('drd_release')->loadByProperties([
      'core' => $this->id(),
    ]);
  }

  /**
   * @inheritDoc
   */
  public function setReleases($releases) {
    $this->set('releases', $releases);
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function getMaintenanceMode($refresh = TRUE) {
    $cache = $this->cacheGet($this->cid('msg'));
    if ($cache) {
      return $cache->data;
    }

    if ($refresh) {
      // We do not know the status, let's get it from remote.
      DrdActionRemoteBase::response('drd_action_maintenance_mode', $this);
      return $this->getMaintenanceMode(FALSE);
    }
    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function getRemoteBlock($module, $delta, $refresh = TRUE) {
    $cache = $this->cacheGet($this->cid(implode(':', ['block', $module, $delta])));
    if ($cache) {
      return $cache->data;
    }

    if ($refresh) {
      DrdActionRemoteBase::response('drd_action_blocks', $this, [
        'module' => $module,
        'delta' => $delta,
      ]);
      return $this->getRemoteBlock($module, $delta, FALSE);
    }
    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function getRequirements($refresh = TRUE) {
    $cache = $this->cacheGet($this->cid('requirements'));
    if ($cache) {
      return $cache->data;
    }

    if ($refresh) {
      DrdActionRemoteBase::response('drd_action_info', $this);
      return $this->getRequirements(FALSE);
    }
    return FALSE;
  }

  // =================== Caching methods =======================================

  /**
   * @param string $topic
   * @return string
   */
  private function cid($topic) {
    return implode(':', ['drd_domain', $this->uuid(), $topic]);
  }

  /**
   * @return array
   */
  private function ctags() {
    return [];
  }

  /**
   * @return int
   */
  private function ctime() {
    return REQUEST_TIME + 600;
  }

  /**
   * @param mixed $text
   * @param bool $translatable
   * @return array|TranslatableMarkup
   */
  private function sanitizeRemoteText($text, $translatable = TRUE) {
    if (is_array($text)) {
      $result = [];
      foreach ($text as $key => $line) {
        $result[$key] = $this->sanitizeRemoteText($line, $translatable);
      }
    }
    elseif (is_string($text)) {
      $text = str_replace('href="/', 'target="_blank" href="'. $this->buildUrl()->toString() .'/', $text);
      $text = str_replace('destination=', 'destoff=', $text);
      if ($translatable) {
        $result = $this->t($text);
      }
      else {
        $result = $text;
      }
    }
    elseif ($text instanceof TranslatableMarkup) {
      // We extract the text so that we can adjust the links.
      $result = $this->sanitizeRemoteText($text->render(), TRUE);
    }
    else {
      $result = $text;
    }

    return $result;
  }

  /**
   * @inheritDoc
   */
  public function cacheRemoteMessages($messages) {
    $cid = $this->cid('msg');
    $cache = $this->cacheGet($cid);
    $data = $cache ? $cache->data : [];
    foreach ($messages as $type => $items) {
      foreach ($items as $item) {
        $data[$type] = [
          'ts' => REQUEST_TIME,
          'msg' => $this->sanitizeRemoteText($item),
        ];
      }
    }
    $this->cacheSet($cid, $data, Cache::PERMANENT, $this->ctags());
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cacheMaintenanceMode($mode) {
    $this->cacheSet($this->cid('maintenancemode'), $mode, $this->ctime(), $this->ctags());
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cacheBlock($module, $delta, $content) {
    $this->cacheSet($this->cid(implode(':', ['block', $module, $delta])), $this->sanitizeRemoteText($content, FALSE), $this->ctime(), $this->ctags());
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cacheErrorLog($log) {
    $this->cacheSet($this->cid('errorlog'), $log, Cache::PERMANENT, $this->ctags());
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cacheRequirements($requirements) {
    foreach ($requirements as $key => $requirement) {
      if (!empty($requirement['value'])) {
        $value = $this->sanitizeRemoteText($requirement['value'], FALSE);
        $requirements[$key]['value'] = is_string($value) ? $this->t($value) : $value;
      }
      if (!empty($requirement['description'])) {
        $description = $this->sanitizeRemoteText($requirement['description'], FALSE);
        $requirements[$key]['description'] = is_string($description) ? $this->t($description) : $description;
      }
    }
    $this->cacheSet($this->cid('requirements'), $requirements, $this->ctime(), $this->ctags());

    // Load the install file, that also loads install.inc from Drupal core.
    \Drupal::moduleHandler()->loadInclude('drd', 'install');
    $warnings = $errors = [];
    // Analyze the requirements and store warnings and errors with the domain.
    foreach ($requirements as $key => $value) {
      $label = isset($value['title']) ? $value['title'] : $key;
      try {
        $requirement = DrdRequirement::findOrCreate($key, $label);
      }
      catch (\Exception $ex) {
        // Ignore old drd_server requirements.
        continue;
      }
      $severity = isset($value['severity']) ? $value['severity'] : REQUIREMENT_INFO;
      switch ($severity) {
        case REQUIREMENT_ERROR:
          $errors[] = $requirement;
          break;

        case REQUIREMENT_WARNING:
          $warnings[] = $requirement;
          break;

      }
    }
    $this->set('warnings', $warnings);
    $this->set('errors', $errors);

    // TODO: Register the related core so that it gets analyzed at the end of the request just once.

    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cacheVariables($variables) {
    $this->cacheSet($this->cid('variables'), $variables, Cache::PERMANENT, $this->ctags());
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cacheGlobals($globals) {
    $this->cacheSet($this->cid('globals'), $globals, Cache::PERMANENT, $this->ctags());
    return $this;
  }

}
