<?php

/**
 * @file
 * Contains \Drupal\drd\Form\DrdCoreForm.
 */

namespace Drupal\drd\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\drd\Crypt\DrdCryptBase;
use Drupal\drd\Entity\DrdCore;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\Language;
use Drupal\drd\Entity\DrdDomain;
use Drupal\drd\Plugin\DrdAuthManager;

/**
 * Form controller for Core edit forms.
 *
 * @ingroup drd
 */
class DrdCoreForm extends ContentEntityForm {
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /* @var $core \Drupal\drd\Entity\DrdCore */
    $form = parent::buildForm($form, $form_state);
    $core = $this->entity;

    $form['langcode'] = [
      '#title' => $this->t('Language'),
      '#type' => 'language_select',
      '#default_value' => $core->langcode->value,
      '#languages' => Language::STATE_ALL,
    ];

    if ($core->isNew()) {

      $form['host']['widget']['#default_value'] = [1];

      // Adding a new core means we need the URL to initially contact that site
      // to grab all the details about that Drupal installation.
      $form['url'] = [
        '#title' => $this->t('URL'),
        '#type' => 'url',
        '#default_value' => '',
        '#required' => TRUE,
        '#ajax' => [
          'callback' => [$this, 'validateUrlAjax'],
          'event' => 'change',
          'progress' => array(
            'type' => 'throbber',
            'message' => t('Verifying url...'),
          ),
        ],
      ];
      $form['url-message'] = [
        '#type' => 'container',
      ];

      // Container for domain specific settings.
      $form['drd'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => 'hidden',
        ],
      ];
      $form['drd'] += DrdAuthManager::authForm($form, $form_state);
      $form['drd'] += DrdCryptBase::cryptForm($form, $form_state);
    }
    else {
      $form['host']['#disabled'] = TRUE;
    }

    return $form;
  }

  /**
   * Validates that the url field points to a drd_remote enabled domain.
   */
  private function validateUrl(array &$form, FormStateInterface $form_state) {
    /** @var DrdCore $core */
    $core = $this->entity;
    $values = DrdAuthManager::authFormValues($form_state) + DrdCryptBase::cryptFormValues($form_state);
    /** @var DrdDomain $domain */
    $domain = DrdDomain::instanceFromUrl($core, $form_state->getValue('url'), $values);

    if (!$domain->isNew()) {
      return $this->t('This domain is already known to the dashboard.');
    }

    $crypt_methods = $domain->getSupportedCryptMethods();
    if ($crypt_methods === FALSE) {
      return $this->t('Can not connect to this domain.');
    }
    if (empty($crypt_methods)) {
      return $this->t('There is no DRD remote module available at this domain.');
    }
    elseif (DrdCryptBase::countAvailableMethods($crypt_methods) == 0) {
      return $this->t('The remote site has DRD installed but does not support any encryption methods matching those of the dashboard.');
    }

    $form_state->setTemporaryValue('drd_domain', $domain);
    return $crypt_methods;
  }

  public function validateUrlAjax(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $crypt_methods = $this->validateUrl($form, $form_state);
    if (is_array($crypt_methods)) {
      $command = 'removeClass';
      $message = '';
      foreach (DrdCryptBase::getMethods() as $key => $value) {
        $response->addCommand(new InvokeCommand('#edit-drd-crypt-type option[value="' . $key . '"]', 'prop', ['disabled', !isset($crypt_methods[$key])]));
      }
    }
    else {
      $command = 'addClass';
      $message = $crypt_methods;
    }
    $response->addCommand(new InvokeCommand('#edit-drd', $command, ['hidden']));
    $response->addCommand(new HtmlCommand('#edit-url-message', $message));
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    /** @var DrdCore $core */
    $core = parent::validateForm($form, $form_state);
    if (!$form_state->hasAnyErrors() && $core->isNew()) {
      $error = $this->validateUrl($form, $form_state);
      if (!empty($error) && is_string($error)) {
        $form_state->setErrorByName('url', $error);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var DrdCore $core */
    $core = $this->entity;
    $status = $core->save();

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label core. ', [
          '%label' => $core->label(),
        ]));

        /** @var DrdDomain $domain */
        $domain = $form_state->getTemporaryValue('drd_domain');
        $domain->setCore($core);
        $domain->save();
        drupal_set_message($this->t('Now you should %configure your remote domain. ', [
          '%configure' => $domain->getRemoteSetupLink($this->t('configure'), TRUE),
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label Core.', [
          '%label' => $core->label(),
        ]));
    }
  }

}
