<?php

/**
 * @file
 * Contains \Drupal\drd\Plugin\DrdActionRemoteBase.
 */

namespace Drupal\drd\Plugin;

use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Session\AccountInterface;
use Drupal\drd\Crypt\DrdCryptBase;
use Drupal\drd\Entity\DrdDomain;
use Drupal\drd\DrdRemoteInterface;
use GuzzleHttp\Client;

/**
 * Base class for DRD Remote Action plugins.
 */
abstract class DrdActionRemoteBase extends DrdActionBase {

  /** @var DrdDomain $domain */
  private $domain;

  protected $response = FALSE;

  /**
   * @param string $id
   * @param DrdRemoteInterface $remote
   *
   * @return array|bool
   */
  public static function response($id, $remote, $arguments = []) {
    /** @var DrdActionRemoteBase $action */
    $action = self::instance($id);
    foreach ($arguments as $key => $value) {
      $action->setActionArgument($key, $value);
    }
    if ($action) {
      return $action->execute($remote);
    }
    return FALSE;
  }

  /**
   * Default action which can and should be overwritten by implementations.
   *
   * {@inheritdoc}
   */
  public function execute($domain = NULL) {
    $this->response = FALSE;
    /** @var DrdDomain $domain */
    if (!$this->access($domain)) {
      return FALSE;
    }
    $this->remoteRequest();
    return $this->getResponse();
  }

  /**
   * {@inheritdoc}
   */
  public function access($domain, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $this->domain = $domain;
    return $return_as_object ? new AccessResultAllowed() : TRUE;

    /**
     * TODO: during cron the current user is anonymous and that would let all
     * actions fail. Let's find out if there is a better way maybe using context
     * to find out if we are called during cron or otherwise.
    $access = $domain->status->access('edit', $account, TRUE)
      ->andIf($domain->access('update', $account, TRUE));

    return $return_as_object ? $access : $access->isAllowed();
    */
  }

  public function getResponse() {
    return $this->response;
  }

  protected function remoteRequest() {
    $url = $this->domain->buildUrl('drd-remote');
    $class = explode('\\', get_class($this));

    // Add authentication
    $args = [
      'auth' => $this->domain->get('auth')->value,
      'authsetting' => $this->domain->getAuthSetting(),
      'action' => array_pop($class),
    ] + $this->arguments;

    // Encrypt the arguments
    $method = $this->domain->get('crypt')->value;
    if (empty($method)) {
      return;
    }
    $crypt = DrdCryptBase::getInstance(
      $method,
      $this->domain->getCryptSetting()
    );

    $body = base64_encode(serialize([
      'uuid' => $this->domain->uuid(),
      'args' => $crypt->encrypt($args),
      'iv' => $crypt->getIV(),
    ]));

    // Send request
    try {
      $client = new Client(['base_uri' => $url->toUriString()]);
      $response = $client->request('get', NULL, ['body' => $body]);
    }
    catch (\Exception $ex) {
      return;
    }
    if ($response->getStatusCode() != 200 || $response->getHeaderLine('content-type') != 'text/plain; charset=utf-8') {
      return;
    }
    $this->response = $crypt->decrypt(base64_decode($response->getBody()->getContents()), $crypt->getIV());

    if (!empty($this->response['messages'])) {
      $this->domain->cacheRemoteMessages($this->response['messages']);
    }
    unset($this->response['messages']);

    if ($followup = $this->getFollowUpAction()) {
      \Drupal::service('queue.drd')->createItem(DrdActionBase::instance($followup), $this->domain);
    }
  }

}
