<?php

/**
 * @file
 * Contains \Drupal\drd\Remote\DrdActionBase.
 */

namespace Drupal\drd\Remote;

/**
 * Base class for Remote DRD Action Code.
 */
abstract class DrdActionBase implements DrdActionInterface {

  private static $debugMode = FALSE;
  private static $arguments = array();

  /**
   * @param $items
   * @return mixed
   */
  private static function toArray($items) {
    foreach ($items as $key => $item) {
      if (is_object($item)) {
        $items[$key] = self::toArray((array) $item);
      }
      elseif (is_array($item)) {
        $items[$key] = self::toArray($item);
      }
    }
    return $items;
  }

  public static function run($version, $debugMode = FALSE) {
    self::setDebugMode($debugMode);
    require_once DRD_BASE . "/Action$version/DrdActionBase.php";
    require_once DRD_BASE . "/Auth$version/DrdAuthBase.php";
    /** @var DrdActionInterface $self */
    $self = "\\Drupal\\drd\\Remote\\Action$version\\DrdActionBase";

    try {
      $raw_input = file_get_contents('php://input');
      if (empty($raw_input)) {
        throw new \Exception('Can not read input');
      }

      $input = unserialize(base64_decode($raw_input));
      if (!is_array($input) || empty($input)) {
        throw new \Exception('Input is empty');
      }

      if (empty($input['uuid']) || empty($input['args']) || !isset($input['iv'])) {
        throw new \Exception('Input is incomplete');
      }

      if (!empty($input['ott']) && !empty($input['config'])) {
        if (!$self::ott($input['ott'], $input['config'])) {
          throw new \Exception('OTT config failed');
        }
        return 'ok';
      }

      $crypt = $self::getCryptInstance($input['uuid']);
      if (!$crypt) {
        throw new \Exception('Encryption method not available or unauthorised');
      }
      $args = self::toArray($crypt->decrypt($input['args'], $input['iv']));

      if (empty($args['auth']) || !isset($args['authsetting']) || empty($args['action'])) {
        throw new \Exception('Arguments incomplete');
      }

      $auth_methods = DrdAuthBase::getMethods($version);
      if (!isset($auth_methods[$args['auth']]) || !($auth_methods[$args['auth']] instanceof DrdAuthInterface)) {
        throw new \Exception('Unrecognized authentication method');
      }

      /** @var DrdAuthInterface $auth */
      $auth = $auth_methods[$args['auth']];
      if (!$auth->validateUUID($input['uuid'])) {
        throw new \Exception('DRD instance not registered');
      }
      if (!$auth->validate($args['authsetting'])) {
        throw new \Exception('Not authenticated');
      }

      $action = $args['action'];
      unset($args['auth']);
      unset($args['authsetting']);
      unset($args['action']);
      self::$arguments = $args;
    }
    catch (\Exception $ex) {
      $self::watchdog($ex->getMessage(), array(), 3);
      header("HTTP/1.1 502 Error");
      print 'error';
      exit;
    }

    try {
      $self::promoteUser();

      require_once DRD_BASE . "/Action$version/$action.php";
      $classname = "\\Drupal\\drd\\Remote\\Action$version\\$action";

      /** @var DrdActionInterface $actionObject */
      $actionObject = new $classname;
    }
    catch (\Exception $ex) {
      $self::watchdog('Not yet implemented: ' . $action, array(), 3);
      header("HTTP/1.1 403 Not found");
      print 'Not yet implemented';
      exit;
    }

    $result = $actionObject->execute();
    $result['messages'] = drupal_get_messages();
    return base64_encode($crypt->encrypt($result));
  }

  /**
   * @inheritDoc
   */
  public static function getArguments() {
    return self::$arguments;
  }

  /**
   * @inheritDoc
   */
  public static function getDebugMode() {
    return self::$debugMode;
  }

  /**
   * @inheritDoc
   */
  public static function setDebugMode($debugMode) {
    self::$debugMode = $debugMode;
  }

}
